# Changelog

Download sensor plugins from <https://hidale.com/shop/>  
Setup and install guide is in file `INSTALL.md`
This software comes with ABSOLUTELY NO WARRANTY.

## v1.3.20240417

* Update to OAK depthai-core v2.25.0 for bug fixes and new OAK hardware
  discovery apis. Update its dependencies with private fixes.
* Fix depthai hardware discovery https://github.com/diablodale/dp.oak/issues/8 yet
  that exposed another depthai bug https://github.com/luxonis/depthai-core/issues/983
* Firmware version query and bootloader update <https://docs.hidale.com/max/methods/firmware>
* Workaround timestamp drifting on some devices for even better frame synchronization
* Add IP address support to `@idsensor` to directly contact that one device instead
  of wide broadcast to all devices on the network
* Change `@mlinput` syntax to be `@mlinput color [fourcc [width height]]`. Its default
  continues to be `@mlinput color rgb`.
* Add machine learning inference example to the helpfile
* Various internal changes for upcoming beta release

## v1.3.20240219

* Fix slow memory leak with ml dictionary output
* Update to Onnxruntime v1.16.3 - memory improvements, caching intermediate models, more ONNX ops
* Optimize color transcoders
* Add `@mlinput` to specify the source and channels using a list of FourCC codes for the input
  tensors for your model. The default `@mlinput rgb` is a single color tensor with 3-channels in
  the order red green blue. Choose the FourCC codes needed for your model. `@mlinput` may be easier
  to understand than `@mlswapch`. For example:
  ```sh
  @mlinput grey     # greyscale, 1-channel, 8-bit integer
  @mlinput bgr      # color, 3-channel in order blue green red, type of @colortype
  @mlinput rgba     # color, 4-channel in order red green blue alpha, type of @colortype
  ```

## v1.3.20231220

* Data processing pipeline simplified, more flexible, less memory use.
* Re-enabled machine learning features for arbitrary onnx models to be run on the colormap.
  See details on `@mlmodel` below in release v1.3.20221118 or at https://docs.hidale.com
* Add `@mount` to indicate how your sensor is mounted (upside up, upside down, left, right).
  Some models (like Azure Body Tracking) may use this to optimize their inference.
* `@decodercolor` is deprecated and replaced by `@transcoder`. `@decodercolor intelmedia` is
  still supported. This is simplier to use. The default is `@transcoder auto` which
  automatically accelerates transcoding/decoding. The only other option is
  `@transcoder intelmedia` which uses the Intel GPU hardware transcoder (also known as QuickSync)
  when possible and then falls back to the `auto` acceleration.
* `uyvy` is better supported...though still not recommended due to widespread Max v6, v7, and v8
  bugs processing and displaying `uyvy` in patches.
* Improved use of OpenCL device for image postprocessing
* Clearer error messages when fatal errors occur
* Fix rare frame desync and large memory use if CPU is consistently overloaded for long periods
* Smaller plugin file size

## v1.3.20231024

* Fix X-axis offset for OAK MoveNet body tracking joint locations; it was ~3.5cm incorrect.
* Much internal refactoring to move more code to reusable components

## v1.3.20230907

* Longer-term solution for Luxonis OAK sensor discovery
* Migrate Intel media SDK to oneAPI VPL
* Update tbb, ipp, depthai, onnx, cuda, cudnn, tensorrt, opencv
* Add warning when playermap setting enabled yet playermap is unavailable

## v1.3.20230628

* Quick workaround for multiple Luxonis OAK sensor discovery bugs; e.g.
  <https://github.com/luxonis/depthai-core/issues/285>,
  <https://github.com/luxonis/depthai-core/issues/862>,
  <https://github.com/luxonis/depthai-core/issues/860>, etc.
* Update toolchain to VS2022

## v1.3.20230614

* Connection improvements for USB OAK devices; minor improvements for PoE.
  Most changes are low-level in the underlying XLink and libusb libraries.

## v1.3.20230414

* Enable color undistort with `@undistort` and `@undistortscale`. Depth is always undistorted.
* Enable skeleton tracking on OAK sensors without IMU

## v1.3.20230320

### Changes

* Renamed incorrect `@colorbalance` to `@whitebalance`.
* Update depthai to v2.20.2
* Update onnxruntime to v1.14.1. Includes fix for caching of TensorRT models.
* Support uncalibrated IMUs
* Add IMU firmware updater

## v1.3.20230119

### Changes

* Remove general machine learning inference `@mlcompute`.
  It was out-of-scope for a sensor plugin.
* Update code that manages attributes and CUDA compute devices

### Known Issues

* Caching of TensorRT models is broken due to Onnx bug
  <https://github.com/microsoft/onnxruntime/issues/14269>. Onnx has their fix
  ready for an upcoming release.

## v1.3.20221216

### Changes

* Update depthai to v2.19.1; requires `libusb-1.0.dll` from ZIP download
* Update onnxruntime to v1.13.1

### Known Issues

* A skeleton tracking joint may have incorrect Z-depth when the sensor's view to that
  specific joint is obscured

## v1.3.20221118

### Changes

* Add skeleton tracking for a maximum of 6 people. Download the model to enable this and
  place it in the same directory as this plugin:
  <https://github.com/diablodale/model-zoo/releases/latest/download/movenet_multipose_lightning_1_int32_op15_H160_W256.onnx>
* Add symbolic device name support to `@opencl`. The numeric index is still supported.
  The symbolic name is any case-insensitive substring of the full device name. For example:
  @opencl off           @opencl default
  @opencl intel         @opencl uhd
  @opencl nvidia        @opencl rtx
  @opencl "2070"  <--- notice the number is quoted as a string because numbers are devices indexes
* Support for Nvidia CUDA compute 3.0 (e.g. GeForce GTX 680) has been removed.
  These GPUs are ten years old and the SDKs themselves have removed support for them.
* Include custom ONNX runtime in the plugin download; fixes onnx bugs not yet merged
  into public onnx distribution
* Add machine learning inference. Expect feature changes and bugs if you use this early feature.
  The ml inference is output on the 5th outlet as a max dictionary. (route) or (routepass) can
  be used to send it to other dict objects for post-processing. Models are restricted to those
  that have a single input that accepts RGB or BGR.
  The color image from the sensor is the source of RGB and is always scaled to the model input's
  size requirements *without* preserving aspect ratio; see `@mlcrop` and `@mlswapch` below.
  SqueezeNet <https://github.com/onnx/models/tree/main/vision/classification/squeezenet> is a
  model I use during development.
  * `@mlmodel` load an ONNX model file and enable the ml feature
  * `@mlcompute` is your choice of directml or cpu device; supports symbolic names
  * `@mlmean` values in channel 0,1,2 (RGB) order are *subtracted* from every pixel `(pixel - mean) * scale`
  * `@mlscale` is multiplied to every pixel `(pixel - mean) * scale`; often used for std deviation
  * `@mlswapch` applies `@mlmean` to channels 2,1,0 *and* swaps data channels 0 and 2
    before sending to the model as input; use when your model requires BGR input
  * `@mlcrop` crops color image with "off", "center", "centerpad", or four integers "xmin ymin xmax ymax"
    ```
    @mlcrop off              resize to input requirement;
                             may change aspect ratio
    @mlcrop center           resize and center crop to input requirement;
                             loose cropped areas, keep aspect ratio 
    @mlcrop centerpad        resize and pad the color image to input requirement;
                             keep aspect ratio
    @mlcrop 160 140 300 350  crop region (columns 160->300 and rows 140->350 all inclusive)
                             then resize to input requirement; may change aspect ratio
    ```
* Fixed setting `@colormap`, `@decodercolor` with symbolic values

### Known Issues

* Enabling `@opencl` on an Intel GPU may crash Max when you first start your sensor. This
  is a known bug with Intel GPU drivers and LLVM (the compiler Intel uses for OpenCL).
  The only workaround is to try again. After clang successfully compiles the OpenCL kernels,
  they will be cached on-disk by OpenCV. These on-disk cached kernels will be used the next
  time and the LLVM clang compiler will not be used; therefore avoiding the crash. This cache
  is not forever and will be purged due to disk usage, Intel driver update, etc. See
  <https://github.com/llvm/llvm-project/issues/29935>, <https://github.com/intel/compute-runtime/issues/122>,
  <https://github.com/NixOS/nixpkgs/issues/97401>.
  For reference, these are the errors that appear in a debugger console during a crash.
  ```
  Loaded 'C:\Windows\System32\DriverStore\FileRepository\iigd_dch.inf_amd64_afa18c61a36f2728\opencl-clang64.dll'. Symbol loading disabled by Include/Exclude setting.
  clang: CommandLine Error: Option 'h' registered more than once!
  LLVM ERROR: inconsistency in registered CommandLine options
  Exception thrown at 0x00007FFC418A79A4 (opencl-clang64.dll) in Max.exe: 0xC0000005: Access violation reading location 0x0000000000000000.
  Exception thrown at 0x00007FFC418A79A4 (opencl-clang64.dll) in Max.exe: 0xC0000005: Access violation reading location 0x0000000000000000.
  Exception thrown at 0x00007FFC418A79A4 (opencl-clang64.dll) in Max.exe: 0xC0000005: Access violation reading location 0x0000000000000000.
  Exception thrown at 0x00007FFC418A79A4 (opencl-clang64.dll) in Max.exe: 0xC0000005: Access violation reading location 0x0000000000000000.
  The program '[665836] Max.exe' has exited with code 1 (0x1).
  ```

## v1.3.20220924

* No public feature changes; almost complete with new body tracking.

## v1.3.20220824

* No public feature changes; only internal work on new features

## v1.3.20220724

* No feature changes; only internal optimizations

## v1.3.20220622

* Use depthai-core v2.15.5; includes firmware fixes
* Add color white balance
* Catch more DepthAI SDK startup errors
* Improve RGB detail and remove aggressive postprocessing

## v1.3.20220521

* Use depthai-core v2.15.4; it includes alignment, calibration, and undistortion fixes
* Update 3rd party library dependencies
* Support for upcoming depthai-core changes

## v1.3.20220422

* Improve accuracy of depth with undistortion data beyond DepthAI SDK.
  If your sensor has consistantly inaccurate depth or unexplained
  curves in flat surfaces, then consider recalibrating your OAK sensor.
  Please note: you do this at your own benefit and/or risk.
  https://docs.luxonis.com/en/latest/pages/calibration/
* Reduce OAK queueing for even less lag
* Add OAK support for UTF-8 paths

## v1.3.20220323

* Initial work on new joints: eyes, ears, nose
* Require align=2 when calculating body skeleton

## v1.3.20220321

* Add OAK device temperature and VPU reporting to Max console
* Tweak joint coordinate, pointcloud formulas
* Improve error reporting to Max console
* Improve frame timestamps

## v1.3.20220310

* Improved frame synchronization
* Registration problems logged to %localappdata%\Dale Phurrough\dp.oak
  for local diagnostics
* CPU and lag performance work
